	TITLE 'BANK & MOVE MODULE FOR CP/M3 LINKED BIOS'

	; DEFINE LOGICAL VALUES:
TRUE		EQU	-1
FALSE		EQU	NOT TRUE

; DETERMINE IF FOR BANK SELECT OR NOT:
; Note - use spaces here instead of tabs so the build script can swap this
; back and forth
BANKED EQU FALSE	

BNKPORT		EQU	036H		;Z80 SBC RELOCATION PORT

; DEFINE PUBLIC LABELS:
PUBLIC	?MOVE,?XMOVE,?BANK

; PUBLIC LABELS AND EXTERNALS FOR BANKED SYSTEMS:
IF	BANKED
	PUBLIC	BANKBUF			;TEMPORARY 128 BYTE BUFFER OTHER ROUTINES
					; MAY USE FOR TRANSIENT PURPOSES
	PUBLIC	DOXMOV,XMOV$BANKS	;EXTENDED BANK MOVE VARIABLES

	; EXTERNALLY DEFINED ENTRY POINTS AND LABELS:
	EXTRN	@CBNK
	EXTRN	?BNKSL
ENDIF

; INCLUDE Z-80 MACROS:
MACLIB	Z80

; LOCATE CODE IN THE COMMON SEGMENT:
CSEG

	;;;;; ?XMOVE
	; ROUTINE SETS UP AN INTER-BANK MOVE OF 128 BYTES ON THE NEXT CALL
	;  TO ?MOVE:
?XMOVE:
IF	BANKED
	MVI	A,0FFH		;SET EXTENDED MOVE FLAG
	STA	DOXMOV
	SBCD	XMOV$BANKS	;AND STORE BANKS FOR THE EXTENDED MOVE
ENDIF
	RET

	;;;;; ?MOVE
	; ROUTINE PERFORMS INTRA-BANK MOVES IF ?XMOVE WAS NOT CALLED PRIOR TO
	;  THIS CALL TO ?MOVE ELSE A 128 BYTE TRANSFER IS CONDUCTED BETWEEN
	;  DIFFERENT BANKS:
?MOVE:
	IF	BANKED
		LDA	DOXMOV		;GET EXTENDED MOVE FLAG
		ORA	A		;IS IT SET ?
		MVI	A,0		;RESET FOR NEXT TIME ANYWAY
		STA	DOXMOV
		JRZ	MOVE$IT

		; HAVE TO IMPLEMENT INTER-BANK MOVE:
		LDA	@CBNK		;REMEMBER CURRENT BANK
		PUSH	PSW
		LDA	XMOV$BANKS	;GET SOURCE BANK
		CALL	?BNKSL		; AND SELECT IT
		PUSH	H		;SAVE DESTINATION ADDRESS
		PUSH	B		;AND THE COUNT
		XCHG			;[HL] = SOURCE
		LXI	D,BANKBUF	;[DE] = LOCAL TEMPORARY BUFFER
		LDIR			;BLOCK MOVE TO TEMP BUFFER
		POP	B		;RESTORE COUNT
		POP	D		;[DE] = ORIGINAL DESTINATION
		PUSH	H		;SAVE NEXT SOURCE ADDRESS
		LXI	H,BANKBUF	;[HL] = SOURCE = TEMP BUFFER
		LDA	XMOV$BANKS+1	;GET DESTINATION BANK
		CALL	?BNKSL		;AND SELECT IT
		LDIR			;BLOCK MOVE FROM TEMP BUFFER TO DESTINATION
		POP	H		;RESTORE NEXT SOURCE
		XCHG			;RETURN SWAP
		POP	PSW		;GET BACK PREVIOUS CURRENT BANK
		JMP	?BNKSL		;SELECT IT AND RETURN

		; ARRIVE HERE FOR INTRA-BANK MEMORY MOVE:
		MOVE$IT:
	ENDIF

	XCHG			;WE ARE PASSED SOURCE IN DE AND DEST IN HL
	LDIR			;USE Z80 BLOCK MOVE INSTRUCTION
	XCHG			;NEED NEXT ADDRESSES IN SAME REGS
	RET

	;;;;; ?BANK
	; ROUTINE SWITCHES IN PHYSICAL BANK:
?BANK:
	IF	BANKED
		CPI	2		;BANK 2 OR HIGHER ?
		JRNC	DONE		;DO NOTHING
		CPI	1		;BANK 1
		JRZ	BNK1$SW		;IF NOT 2 OR HIGHER OR 1, MUST BE ZERO
		MVI	A,0
		OUT	BNKPORT
		DONE:	
			RET

		BNK1$SW:
			MVI	A,1	;SWITCH TO BANK 1
			OUT 	BNKPORT
	ENDIF
	RET

IF	BANKED
	DOXMOV:
		DB	0	;EXTENDED MOVE FLAG -- IF EQUAL TO FFH THEN
				; NEXT CALL TO ?MOVE WILL BE AN INTER-BANK
				; MOVE.
	XMOV$BANKS:
		DS	1	;STORAGE AREA FOR DESTINATION BANK # FOR
				; EXTENDED MOVES
		DS	1	;STORAGE AREA FOR SOURCE BANK # FOR
				; EXTENDED MOVES

	BANKBUF:
		DS	128	;LOCAL TEMPORARY BUFFER FOR EXTENDED MOVES
ENDIF

END
